/*
	Application.h
	Application Kit, Release 2.0
	Copyright (c) 1988, 1989, 1990, NeXT, Inc.  All rights reserved. 
*/

#import "NSResponder.h"
#import "NSPasteboard.h"
#import "screens.h"
#import "NSDataLinkManager.h"
#import "workspaceRequest.h"
#import <objc/hashtable.h>
#import <remote/NXConnection.h>
@class NSDataLinkManager;

extern NSString *NSModalPanelRunLoopMode;
extern NSString *NSEventTrackingRunLoopMode;

enum {
    NSRunStoppedResponse			= (-1000),
    NSRunAbortedResponse			= (-1001),
    NSRunContinuesResponse			= (-1002)
};

extern id NSApp;
extern NSString *NSSystemDomainName;

extern id NXGetNamedObject(const char *name, id owner);
extern const char *NXGetObjectName(id theObject);
extern int NXNameObject(const char *name, id theObject, id owner);
extern int NXUnnameObject(const char *name, id owner);

extern BOOL NXUserAborted(void);
extern void NXResetUserAbort(void);

typedef struct _NXModalSession {
    id                      app;
    id                      window;
    struct _NXModalSession *prevSession;
    int                     oldRunningCount;
    BOOL                    oldDoesHide;
    BOOL                    freeMe;
    int                     winNum;
    NXHandler              *errorData;
    DPSContext              ctxt;
    int                     reserved2;
}  NSModalSession;

@class NSEvent;

@interface NSApplication : NSResponder
{
    NSString            *appName;
    NSEvent *currentEvent;
    id                  windowList;
    id                  keyWindow;
    id                  mainWindow;
    id                  delegate;
    int                *hiddenList;
    int                 hiddenCount;
    NSString           *hostName;
    DPSContext          context;
    int                 contextNum;
    id                  appListener;
    id                  appSpeaker;
    port_t              replyPort;
    NSSize              screenSize;
    short               running;
    struct __appFlags {
	unsigned int        hidden:1;
	unsigned int        autoupdate:1;
	unsigned int        active:1;
	unsigned int        _hasBeenRun:1;
	unsigned int        _RESERVED:7;
	unsigned int        _doingUnhide:1;
	unsigned int	    _delegateReturnsValidRequestor:1;
	unsigned int	    _deactPending:1;
	unsigned int        _invalidState:1;
	unsigned int        _invalidEvent:1;
    }                   appFlags;
    id                  _reservedApp4;
    id                  _focusStack;
    id                  _freelist;
    id                  _pboard;
    id                  _mainMenu;
    id                  _appIcon;
    id                  _nameTable;
    id                  _printInfo;
    unsigned int        _reservedApp1;
    unsigned int        _reservedApp2;
    unsigned int        _reservedApp3;
}

+ (id <NXWorkspaceRequestProtocol>)workspace;

+ initialize;
+ sharedApplication;

- setDelegate:anObject;
- delegate;
- (DPSContext)context;
- hide:sender;
- unhide:sender;
- unhideWithoutActivation:sender;
- findWindow:(int)windowNum;
- focusView;
- mainWindow;
- keyWindow;
- (port_t)replyPort;
- (NSString *)appName;
- (const char *const *)systemLanguages;
- (NSString *)appListenerPortName;
- appListener;
- setAppListener:aListener;
- appSpeaker;
- setAppSpeaker:aSpeaker;
- (int)unhide;
- (int)openFile:(NSString *)fullPath ok:(int *)flag;
- (int)openTempFile:(NSString *)fullPath ok:(int *)flag;
- (int)powerOffIn:(int)ms andSave:(int)aFlag;
- (int)unmounting:(NSString *)fullPath ok:(int *)flag;
- (int)unmounted:(NSString *)fullPath;
- (int)mounted:(NSString *)fullPath;
- (int)fileOperationCompleted:(int)operation;
- (int)applicationWillLaunch:(NSString *)applicationName;
- (int)applicationDidLaunch:(NSString *)applicationName;
- (int)applicationDidTerminate:(NSString *)applicationName;
- (BOOL)isActive;
- (NSString *)hostName;
- (BOOL)isHidden;
- (BOOL)isRunning;
- (int)activeApp;
- deactivateSelf;
- (int)activateSelf:(BOOL)flag;
- (int)activate:(int)contextNumber;
- (void)run;
- (int)runModalFor:theWindow;
- stop:sender;
- stopModal;
- stopModal:(int)returnCode;
- (void)abortModal;
- (NSModalSession *)beginModalSession:(NSModalSession *)session for:theWindow;
- (int)runModalSession:(NSModalSession *)session;
- endModalSession:(NSModalSession *)session;
- setAutoupdate:(BOOL)flag;
- terminate:sender;
- (NSEvent *)nextEventMatchingMask:(unsigned int)mask untilDate:(NSDate *)expiration inMode:(NSString *)mode dequeue:(BOOL)deqFlag;
- (void)discardEventsMatchingMask:(unsigned int)mask beforeEvent:(NSEvent *)lastEvent;
- (void)postEvent:(NSEvent *)event atStart:(BOOL)flag;
- (NSEvent *)currentEvent;

- sendEvent:(NSEvent *)theEvent;
- powerOff:(NSEvent *)theEvent;
- applicationDefined:(NSEvent *)theEvent;
- rightMouseDown:(NSEvent *)theEvent;
- preventWindowOrdering;
- becomeActiveApp;
- resignActiveApp;
- makeWindowsPerform:(SEL)aSelector inOrder:(BOOL)flag;
- appIcon;
- windowList;
- getWindowNumbers:(int **)list count:(int *)winCount;
- updateWindows;
- (BOOL)sendAction:(SEL)theAction to:theTarget from:sender;
- calcTargetForAction:(SEL)theAction;
- (BOOL)tryToPerform:(SEL)anAction with:anObject;
- printInfo;
- runPageLayout:sender;
- orderFrontColorPanel:sender;
- setMainMenu:aMenu;
- mainMenu;
- delayedFree:theObject;
- getScreens:(const NXScreen **)list count:(int *)count;
- (const NXScreen *)mainScreen;
- (const NXScreen *)colorScreen;
- (NSSize)screenSize;
- loadNibFile:(NSString *)fileName owner:anOwner withNames:(BOOL)aFlag fromZone:(NSZone *)zone;
- loadNibFile:(NSString *)fileName owner:anOwner withNames:(BOOL)aFlag;
- loadNibFile:(NSString *)fileName owner:anOwner;
- loadNibSection:(NSString *)sectionName owner:anOwner withNames:(BOOL)aFlag;
- loadNibSection:(NSString *)sectionName owner:anOwner withNames:(BOOL)aFlag fromZone:(NSZone *)zone;
- loadNibSection:(NSString *)sectionName owner:anOwner withNames:(BOOL)flag fromHeader:(const struct mach_header *)mhp;
- loadNibSection:(NSString *)sectionName owner:anOwner withNames:(BOOL)flag fromHeader:(const struct mach_header *)mhp fromZone:(NSZone *)zone;
- loadNibSection:(NSString *)sectionName owner:anOwner;
- validRequestorForSendType:(NSString *)sendType andReturnType:(NSString *)returnType;
- slaveJournaler;
- masterJournaler;
- (BOOL)isJournalable;
- setJournalable:(BOOL)newStatus;
- showHelpPanel:sender;

@end

@interface NSApplication(WindowsMenu)
- windowsMenu;
- setWindowsMenu:(id)menu;
- arrangeInFront:sender;
- removeWindowsItem:(id)win;
- addWindowsItem:(id)win title:(NSString *)aString filename:(BOOL)isFilename;
- changeWindowsItem:(id)win title:(NSString *)aString filename:(BOOL)isFilename;
- updateWindowsItem:(id)win;
- miniaturizeAll:sender;
@end


@interface NSObject(ApplicationDelegate)
- appWillInit:sender;
- appDidInit:sender;
- appDidHide:sender;
- appDidUnhide:sender;
- appDidBecomeActive:sender;
- appDidResignActive:sender;
- appDidUpdate:sender;
- appWillUpdate:sender;
- appWillTerminate:sender;
- app:sender willShowHelpPanel:panel;
- (BOOL)appAcceptsAnotherFile:sender;
- (int)app:sender openFile:(NSString *)filename type:(NSString *)aType;
- (int)app:sender openTempFile:(NSString *)filename type:(NSString *)aType;
- (NSDataLinkManager *)app:sender openFileWithoutUI:(NSString *)filename type:(NSString *)aType;
- app:sender powerOffIn:(int)ms andSave:(int)aFlag;
- (int)app:sender unmounting:(NSString *)fullPath;
- app:sender unmounted:(NSString *)fullPath;
- app:sender mounted:(NSString *)fullPath;
- app:sender fileOperationCompleted:(int)operation;
- app:sender applicationWillLaunch:(NSString *)appName;
- app:sender applicationDidLaunch:(NSString *)appName;
- app:sender applicationDidTerminate:(NSString *)appName;
- applicationDefined:(NSEvent *)theEvent;
- powerOff:(NSEvent *)theEvent;
@end

@interface NSApplication(ServicesMenu)
- servicesMenu;
- setServicesMenu:aMenu;
- registerServicesMenuSendTypes:(NSArray *)sendTypes andReturnTypes:(NSArray *)returnTypes;
@end

@interface NSApplication(NXAlphaImport) 
- setImportAlpha:(BOOL)flag;
- (BOOL)doesImportAlpha;
@end

@interface NSObject(NXServicesRequests)
- (BOOL)writeSelectionToPasteboard:pboard types:(NSArray *)types;
- readSelectionFromPasteboard:pboard;
@end

@interface NSObject(DelayedPerform)
- perform:(SEL)aSelector with:anArg afterDelay:(int)ms cancelPrevious:(BOOL)flag;
@end

@interface NSObject(NXNibNotification)
- awakeFromNib;
@end

@interface NXConnection(NXConnectionHelper) 
- runFromAppKit;
- runFromAppKitWithPriority:(int)priority;
@end


#ifdef KANJI
@interface NSApplication(InputManager)
- inputManager;
- setInputManager: newIM;
@end
#endif
