/*
	FontManager.h
	Application Kit, Release 2.0
	Copyright (c) 1988, 1989, 1990, NeXT, Inc.  All rights reserved. 
*/

#import <Foundation/NSObject.h>

@class NSFontPanel;
@class NSFont;
@class NSMenu;
@class NSArray;

typedef unsigned int NSFontTraitMask;

/*
 * Font Traits
 *
 * This list should be kept small since the more traits that are assigned
 * to a given font, the harder it will be to map it to some other family.
 * Some traits are mutually exclusive such as NSExpanded and NSCondensed.
 */

enum {
    NSItalicFontMask			= 0x00000001,
    NSBoldFontMask			= 0x00000002,
    NSUnboldFontMask			= 0x00000004,
    NSNonStandardCharacterSetFontMask	= 0x00000008,
    NSNarrowFontMask			= 0x00000010,
    NSExpandedFontMask			= 0x00000020,
    NSCondensedFontMask			= 0x00000040,
    NSSmallCapsFontMask			= 0x00000080,
    NSPosterFontMask			= 0x00000100,
    NSCompressedFontMask		= 0x00000200,
    NSUnitalicFontMask			= 0x01000000
};

@interface NSFontManager : NSObject
{
    id                  panel;
    id                  menu;
    SEL                 action;
    int                 whatToDo;
    NSFontTraitMask     traitToChange;
    id                  selFont;
    struct _fmFlags {
	unsigned int        multipleFont:1;
	unsigned int        disabled:1;
	unsigned int        _RESERVED:14;
    }                   fmFlags;
    unsigned short      _lastPos;
    id		        delegate;
    unsigned int        _reservedFMint2;
    unsigned int        _reservedFMint3;
    unsigned int        _reservedFMint4;
}

+ setFontPanelFactory:factoryId;
+ setFontManagerFactory:factoryId;
+ (NSFontManager *)sharedFontManager;

- (BOOL)isMultiple;
- selectedFont;
- setSelectedFont:(NSFont *)fontObj isMultiple:(BOOL)flag;
- (void)setFontMenu:(NSMenu *)newMenu;
- fontMenu:(BOOL)create;
- fontPanel:(BOOL)create;
- (NSFont *)fontWithFamily:(NSString *)family traits:(NSFontTraitMask)traits weight:(int)weight size:(float)size;
- (NSFontTraitMask)traitsOfFont:(NSFont *)fontObj;
- (int)weightOfFont:(NSFont *)fontObj;
- (NSArray *)availableFonts;
- convertFont:fontObj toSize:(float)size;
- convertFont:fontObj toFace:(NSString *)typeface;
- convertFont:fontObj toFamily:(NSString *)family;
- convertFont:fontObj toHaveTrait:(NSFontTraitMask)trait;
- (NSFont *)convertFont:(NSFont *)fontObj toNotHaveTrait:(NSFontTraitMask)trait;
- convertWeight:(BOOL)upFlag ofFont:fontObj;

- (BOOL)isEnabled;
- setEnabled:(BOOL)flag;
- (SEL)action;
- setAction:(SEL)aSelector;
- sendAction;
- setDelegate:anObject;
- delegate;

@end

@interface NSObject(FontManagerDelegate)
- (BOOL)fontManager:sender willIncludeFont:(NSString *)fontName;
@end
